/***************************************************************************
                          exserdoc.cpp  -  description
                             -------------------
    begin                : Tue Jun 26 09:42:14 PDT 2001
    copyright            : (C) 2001 by root
    email                : root@localhost.localdomain
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// ANSI standard include files */
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
#include <sys/ioctl.h>
#include <stdlib.h>
#include <linux/serial.h>
#include <linux/serial_reg.h>

// include files for Qt
#include <qdir.h>
#include <qfileinfo.h>
#include <qwidget.h>
#include <qmsgbox.h>
#include <qfiledialog.h>
#include <qsocketnotifier.h>
#include <qcombobox.h>
#include <qradiobutton.h>
#include <qcheckbox.h>


// application specific includes
#include "exserdoc.h"
#include "exser.h"
#include "exserview.h"
#include "serparmdlg.h"
#include "sertestdlg.h"

/* Serial port driver specific defines */
#define EFR_850	2		//16C850 EFR
#define TRG_850	0	// 16c850
#define XON1_850 4
#define XON2_850	5
#define XOFF1_850	6
#define XOFF2_850 7

#define EMSR_750	7

#define FCR	2	// 16C650/750/850 special function register
#define XFR	5		// 16C650/750/850 special function register
#define IRPW	6 // 16C650/750/850 special function register
#define FCTR	1	// 16C650/750/850 special function register


#define FCTR_158 8
#define EFR_158 9
#define TXCNT_158 10
#define TXTRG_158 10
#define RXCNT_158 11
#define RXTRG_158 11
#define XON1_158	14
#define XOFF1_158	12

#define XON		0x11
#define XOFF	0x13

/* Flow control */
#define FLOW_NONE 0
#define FLOW_XONXOFF 1
#define FLOW_RTSCTS 2
#define FLOW_DTRDSR 3

#define EXSER_READ_REG (FIOQSIZE + 1)
#define EXSER_WRITE_REG (FIOQSIZE + 2)

/* EXSER DOC defines */
#define SERIAL_BUF_SIZE 5000
#define QUICK_BROWN 1
#define ALPHABETS 2
#define USER_PAT	3
#define FILE_PAT	4

#define ST16C550 PORT_16550A
#define ST16C650 PORT_16650V2
#define XR16C750 PORT_16750
#define XR16C850 PORT_16850
#define XR17C158 (PORT_MAX + 1)

char QuickBrown[] = "Quick Brown Fox Jumps Over The Lazy Dog\r\n";
char Alphabet[]		= "0123456789abcdefghijklmnopqrstuvwxyz \r\n";

ExserDoc::ExserDoc()
{
  pViewList = new QList<ExserView>;
  pViewList->setAutoDelete(false);

  serfd = TimerID = TestingFlg = testfd = -1;
  noBytes = noTx = 0;
  rdBuf = new char [SERIAL_BUF_SIZE];
  baudrate = 10; databits = 3; parity = 4;
	txtriglvl = rxtriglvl = stopbits = 0;
  dtrhystflg = rtshystflg = dtrhyst = rtshyst = autorsflg = intloopb = flowcontrol = 0;
  txfifoflg = rxfifoflg = 0;
  packerr = StopOnErrFlg = NoPackFlg = NoPack = 0;ContFlg = 1;
	testpat = QUICK_BROWN;

  TxBuf = new char [100];
  RxBuf = new char [100];
	
  /* Initialize for the test pattern */
	PatLen = sizeof( QuickBrown)  - 1;
	memcpy( TxBuf, QuickBrown, sizeof( QuickBrown) );	
}

ExserDoc::~ExserDoc()
{	
	int tmpfd;
	QString inifile(QString(m_title+".ini"));
	
	
	if( -1 == ( tmpfd = open( inifile, O_CREAT ) ) ){
		QMessageBox::information( 0, inifile, "Unable to save the parameters");
	}
	
	else {
		close( tmpfd );
	
		/* Read the initiali parameters from the file */
	  QFile fini( inifile );
	  if( fini.open( IO_WriteOnly ) ){
		  // open file for writing sucess
  		QDataStream s( &fini );                // serialize using f
	  	s << baudrate;                // serialize integer
  	 	s << databits;
	  	s << parity;
		  s << stopbits;
  	  s << flowcontrol;
   		s << dtrhystflg;
		  s << rtshystflg;
  	  s << dtrhyst;
   		s << rtshyst;
		  s << autorsflg;
  	  s << ContFlg;
   		s << txfifoflg;
		  s << rxfifoflg;
  	  s << txtriglvl;
   		s << rxtriglvl;
  	  s << StopOnErrFlg;
   		s << intloopb;
		  s << NoPackFlg;
  	  s << NoPack;
   		fini.close();
	  }
  }
  	
	if( TimerID != -1)killTimer( TimerID );
	delete rdBuf;
	delete TxBuf;
	delete RxBuf;
  delete pViewList;
}

void ExserDoc::addView(ExserView *view)
{
  pViewList->append(view);
	changedViewList();
}

void ExserDoc::removeView(ExserView *view)
{
	  pViewList->remove(view);
	  if(!pViewList->isEmpty())
			changedViewList();
		else
			deleteContents();
}

void ExserDoc::changedViewList(){	
	
	ExserView *w;
	if((int)pViewList->count() == 1){
  	w=pViewList->first();
  	w->setCaption(m_title);
	}
	else{
		int i;
    for( i=1,w=pViewList->first(); w!=0; i++, w=pViewList->next())
  		w->setCaption(QString(m_title+":%1").arg(i));	
	}
}

bool ExserDoc::isLastView() {
  return ((int) pViewList->count() == 1);
}


void ExserDoc::updateAllViews(ExserView *sender)
{
  ExserView *w;
  for(w=pViewList->first(); w!=0; w=pViewList->next())
  {
     w->update(sender);
  }

}

void ExserDoc::setPathName(const QString &name)
{
  m_filename=name;
	m_title=QFileInfo(name).fileName();
}

const QString& ExserDoc::pathName() const
{
  return m_filename;
}

void ExserDoc::setTitle(const QString &title)
{
  m_title=title;
}

const QString &ExserDoc::title() const
{
  return m_title;
}


void ExserDoc::closeDocument()
{
  ExserView *w;

  if(!isLastView())
  {
    for(w=pViewList->first(); w!=0; w=pViewList->next())
    {
   	 	if(!w->close())
 				break;
    }
	}
  if(isLastView())
  {
  	w=pViewList->first();
  	w->close();
  }
      	
  if( serfd != -1)close( serfd );
}

bool ExserDoc::newDocument()
{
  /////////////////////////////////////////////////
  // TODO: Add your document initialization code here
  /////////////////////////////////////////////////
  modified=false;
  return true;
}

bool ExserDoc::openDocument(const QString &filename, const char *format /*=0*/)
{
	struct exser_read_reg exser_reg;
	int lcr;

	struct serial_struct serialst;

	QFile f( filename );
	
	QString inifile(QFileInfo(f).fileName() + ".ini" ) ;
	
	if( -1 == ( serfd = open( filename, O_RDWR | O_NOCTTY ))){
		return false;
	}

  QSocketNotifier *sn;
  sn = new QSocketNotifier( serfd, QSocketNotifier::Read, this );
  QObject::connect( sn, SIGNAL(activated(int)),
                    this, SLOT(ComRead()) );

 	/* get the serial port structure */
	ioctl( serfd, TIOCGSERIAL, &serialst );
	UARTType = serialst.type;
	
	/* Check for the XR17C158 */
	if( UARTType == PORT_16550A && serialst.port > 0x10000)UARTType = XR17C158;
#if 0
	else {
		/* Read the DLM to get the device ID */
		/* Access special function registers */
		exser_reg.reg = UART_LCR;
		ioctl(serfd, EXSER_READ_REG, &exser_reg);
		lcr = exser_reg.regvalue;		/* store the previous LCR reg. value */
		
		exser_reg.regvalue = UART_LCR_DLAB;
		ioctl( serfd, EXSER_WRITE_REG, &exser_reg);
		
		/* write DLL and DLM 0s */
		exser_reg.reg = UART_DLL;
		exser_reg.regvalue = 0;
		ioctl( serfd, EXSER_WRITE_REG, &exser_reg );
		
		exser_reg.reg = UART_DLM;
		exser_reg.regvalue = 0;
		ioctl( serfd, EXSER_WRITE_REG, &exser_reg );
	
		/* Read the device ID */	
		ioctl( serfd, EXSER_READ_REG, &exser_reg );
		
		switch( exser_reg.regvalue) {
			case 4:	/* XR  ST16650 */
				UARTType = PORT_16650V2;
				break;
			case 8:
			case 0xa:
				UARTType = PORT_16750;
				break;
			case 0x10:
			case 0x12:
			case 0x14:
				UARTType = PORT_16850;
				break;
				
			default:
				break;
		}
		
		exser_reg.reg = UART_LCR;
		exser_reg.regvalue = lcr;
		ioctl( serfd, EXSER_WRITE_REG, &exser_reg );
	}
					
#endif	
	
	
  /* Read the initiali parameters from the file */
	QFile fini( inifile );
  if( fini.open( IO_ReadOnly ) ){
	  // open file for Reading sucess
	  QDataStream s( &fini );                // serialize using f
    s >> baudrate;                // serialize integer
	  s >> databits;
    s >> parity;
   	s >> stopbits;
	  s >> flowcontrol;
    s >> dtrhystflg;
   	s >> rtshystflg;
	  s >> dtrhyst;
    s >> rtshyst;
   	s >> autorsflg;
	  s >> ContFlg;
    s >> txfifoflg;
   	s >> rxfifoflg;
	  s >> txtriglvl;
    s >> rxtriglvl;
	  s >> StopOnErrFlg;
    s >> intloopb;
   	s >> NoPackFlg;
	  s >> NoPack;
    fini.close();
	}
	else {
		switch( UARTType ){
			case PORT_16550A:
				txtriglvl = 0; rxtriglvl = 8;
				break;
			
			case ST16C650:
				txtriglvl = 2; rxtriglvl = 2;
				break;
			
			case XR16C750:
				txtriglvl = 8; rxtriglvl = 47;
				break;
			
			case XR16C850:
				txtriglvl = 8; rxtriglvl = 100;
				break;
			
			case XR17C158:
				txtriglvl = 8; rxtriglvl = 47;
				break;
			
			default:
				txtriglvl = rxtriglvl = 0;
				break;
		}
	}
	
	if( ! SetSerialParm())	return false;
		
/*	if ( !f.open( IO_ReadOnly ) )
		return false;
  /////////////////////////////////////////////////
  // TODO: Add your document opening code here
  /////////////////////////////////////////////////
	f.close();
	*/
  modified=false;
  m_filename=filename;
	m_title=QFileInfo(f).fileName();
  return true;
}

bool ExserDoc::saveDocument(const QString &filename, const char *format /*=0*/)
{
	QFile f( filename );
	if ( !f.open( IO_WriteOnly ) )
		return false;

  /////////////////////////////////////////////////
  // TODO: Add your document saving code here
  /////////////////////////////////////////////////

  f.close();

  modified=false;
  m_filename=filename;
	m_title=QFileInfo(f).fileName();
  return true;
}

void ExserDoc::deleteContents()
{
  /////////////////////////////////////////////////
  // TODO: Add implementation to delete the document contents
  /////////////////////////////////////////////////

}

bool ExserDoc::canCloseFrame(ExserView* pFrame)
{
	if(!isLastView())
		return true;
		
	bool ret=false;
  if(isModified())
  {
		QString saveName;
  	switch(QMessageBox::information(pFrame, title(), tr("The current file has been modified.\n"
                          "Do you want to save it?"),QMessageBox::Yes, QMessageBox::No, QMessageBox::Cancel ))
    {
			case QMessageBox::Yes:
				if(title().contains(tr("Untitled")))
				{
					saveName=QFileDialog::getSaveFileName(0, 0, pFrame);
                                          if(saveName.isEmpty())
          	                                return false;
				}
				else
					saveName=pathName();

				if(!saveDocument(saveName))
				{
 					switch(QMessageBox::critical(pFrame, tr("I/O Error !"), tr("Could not save the current document !\n"
																												"Close anyway ?"),QMessageBox::Yes ,QMessageBox::No))

 					{
 						case QMessageBox::Yes:
 							ret=true;
 						case QMessageBox::No:
 							ret=false;
 					}
				}
				else
					ret=true;
				break;
			case QMessageBox::No:
				ret=true;
				break;
			case QMessageBox::Cancel:
			default:
				ret=false;
				break;
		}
	}
	else
		ret=true;

	return ret;
}

int iabaudrate[] = { B110, B300, B600, B1200, B2400, B4800, B9600,
										 B19200, B38400, B57600, B115200, B230400, B460800, B921600};
int iadatasize[] = { CS5, CS6, CS7, CS8 };

int iaparityi[]	 = { 0, 0, PARMRK, 0, IGNPAR };
int	iaparityc[]  = { PARODD, PARENB, 0, 0, 0};

int iastopbits[] = { 0, CSTOPB };

int	iaflowcontroli[] = { 0, IXON | IXOFF, 0, 0};
int	iaflowcontrolc[] = { 0, 0, CRTSCTS, CRTSCTS };

int iahyst[]	= { 0, 4, 6, 8, 16 };
										
bool ExserDoc::SetSerialParm()
{
	struct exser_read_reg exser_reg;
	struct termios tattr;
	
	/* Initialize the serial port with the required parameters */
	bzero( &tattr, sizeof(tattr));
/*	
	tattr.c_cflag = B38400 | CLOCAL | CS8 | CREAD | CRTSCTS;
	tattr.c_iflag = IGNPAR;
	tattr.c_oflag = 0;
	tattr.c_lflag = 0;
*/
	
	tattr.c_iflag = iaparityi[ parity ] | iaflowcontroli[ flowcontrol ];
	tattr.c_oflag = 0;
	tattr.c_cflag = iadatasize[databits] | iastopbits[stopbits] |
									CREAD | iaparityc[parity] | CLOCAL |
									iaflowcontrolc[flowcontrol] |
									iabaudrate[baudrate];
	tattr.c_lflag = 0;
	
	if( tcflush( serfd, TCIFLUSH))return false;
	
	if( tcsetattr( serfd, TCSANOW, &tattr)) return false;
	
	/* Set internal local loopback */
	exser_reg.reg = UART_MCR;
	ioctl(serfd, EXSER_READ_REG, &exser_reg);
	if( intloopb){
		exser_reg.regvalue |= 0x10;
	}
	else {
		exser_reg.regvalue &= 0xef;
	}
  ioctl(serfd, EXSER_WRITE_REG, &exser_reg);

#if 0
	ioctl (serfd, EXSER_READ_REG, &exser_reg);
	if (exser_reg.regvalue & 0x10) {
	QMessageBox::information(0, "Exser", "Internal Loopback Enabled");
	}	
#endif

	switch( UARTType ){
		case PORT_16550A:
			ST16C550Init();
			break;
			
		case ST16C650:
			ST16C650Init();
			break;
			
		case XR16C750:
			XR16C750Init();
			break;
			
		case XR16C850:
//			XR16C850Init();
			break;
		
		case XR17C158:
			XR17C158Init();
			break;
			
		default:
			/* Assuming the 8250. No special function regs. */
			/* Nothing to Initialize */
			QMessageBox::information(0, "Exser", "Initializing 8250 UART");

			break;
	}

	return true;
}


void ExserDoc::ST16C550Init()
{
	struct exser_read_reg exser_reg;
		
	QMessageBox::information(0, "Exser", "Initializing ST16C550 UART");
	
	if( rxfifoflg ){
		exser_reg.reg = FCR;
		ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read FCR contents */
		exser_reg.regvalue &= 0x3f;		/* Clear the RX trig level bits 6, 7 */
		exser_reg.regvalue |= 1;			/* Enable transmit and receive FIFO */
		exser_reg.regvalue |= rxtriglvl << 6;
		ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
	}
}

void ExserDoc::ST16C650Init()
{
	struct exser_read_reg exser_reg;
	int lcr;
		
	QMessageBox::information(0, "Exser", "Initializing ST16C650A UART");

		/* Set Auto RS485 Enable */
	exser_reg.reg = XFR;
	ioctl(serfd, EXSER_READ_REG, &exser_reg);
	
	if( autorsflg ){
		exser_reg.regvalue |= 0x4;
	}
	else {
		exser_reg.regvalue &= 0xfb;
	}
  ioctl(serfd, EXSER_WRITE_REG, &exser_reg);

	/* Access special function registers */
	exser_reg.reg = UART_LCR;
	ioctl(serfd, EXSER_READ_REG, &exser_reg);
	lcr = exser_reg.regvalue;		/* store the previous LCR reg. value */
	
	exser_reg.regvalue = 0xbf;	/* Access the special function registers */
	ioctl(serfd, EXSER_WRITE_REG, &exser_reg);

  if( flowcontrol == FLOW_RTSCTS || flowcontrol == FLOW_NONE ){
		
		/* Set hardware handshaking */
		if( txfifoflg){
			exser_reg.reg = FCR;
			ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read FCR contents */
			exser_reg.regvalue &= 0xcf;		/* Clear the TX trig level bits 4, 5 */
			exser_reg.regvalue |= txtriglvl << 4;
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		}
		
		if( rxfifoflg ){
			exser_reg.reg = FCR;
			ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read FCR contents */
			exser_reg.regvalue &= 0x3f;		/* Clear the TX trig level bits 6, 7 */
			exser_reg.regvalue |= rxtriglvl << 6;
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		}
		
	}

#if 0	

	/* couldnot find the correct registers */

	if( flowcontrol == FLOW_XONXOFF ){
	/* Set XON/XOFF trigger and hysterisis */
		exser_reg.reg = EFR_850;
		ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read EFR contents */
		exser_reg.regvalue &= 0x3f;		/* Disable hardware handshake */
		exser_reg.regvalue |= 0xa;		/* Enable Tx/Rx XON1/XOFF1 */		
		ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		
		exser_reg.reg = XON1_850;
		exser_reg.regvalue = XON;		/* Set XON1 */		
		ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		
		exser_reg.reg = XOFF1_850;
		exser_reg.regvalue = XOFF;		/* Set XOFF1 */		
		ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		
		if(txfifoflg | rxfifoflg){
			/* Set the trigger levels */
			exser_reg.reg = FCTR_850;
			ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read FCTR contents */
			exser_reg.regvalue |= 0x98;		/* Table D selecet, Tx prog. trig level */
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
			
			if( txfifoflg){
				exser_reg.reg = TRG_850;
				exser_reg.regvalue = txtriglvl;		/* Tx trigger level from dialog box */
				ioctl(serfd, EXSER_WRITE_REG, &exser_reg);				
			}
			
			if( rxfifoflg ){
				exser_reg.reg = FCTR_850;
				ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read FCTR contents */
				exser_reg.regvalue |= 0x98;		/* Table D select, Tx prog. trig level */
				exser_reg.regvalue &= 0x7f;		/* Table D select, Rx prog. trig level */
				ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
			
				exser_reg.reg = TRG_850;
				exser_reg.regvalue = rxtriglvl;		/* Rx trigger level from dialog box */
				ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
			}
		}
	
		/* Set the Hysterisis */
		exser_reg.reg = FCTR_850;
		ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read FCTR contents */
		exser_reg.regvalue &= 0xfc ;		/* clear the hyst */
		
		if( (flowcontrol == 0) && rtshystflg){
			exser_reg.regvalue |= rtshyst ;		/* set the hyst */
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		}	
	}
#endif
		
	exser_reg.reg = UART_LCR;
	exser_reg.regvalue = lcr;
	ioctl(serfd, EXSER_WRITE_REG, &exser_reg);	/* Access the General registers */
	
}

void ExserDoc::XR16C750Init()
{
	struct exser_read_reg exser_reg;
	int lcr;
		
	QMessageBox::information(0, "Exser", "Initializing XR16C750 UART");

	/* Access special function registers */
	exser_reg.reg = UART_LCR;
	ioctl(serfd, EXSER_READ_REG, &exser_reg);
	lcr = exser_reg.regvalue;		/* store the previous LCR reg. value */
	
	exser_reg.regvalue = 0xbf;	/* Access the special function registers */
	ioctl(serfd, EXSER_WRITE_REG, &exser_reg);

	/* Set Auto RS485 Enable */
	exser_reg.reg = FCTR;
	ioctl(serfd, EXSER_READ_REG, &exser_reg);
	if( autorsflg ){
		exser_reg.regvalue |= 0x8;
	}
	else {
		exser_reg.regvalue &= 0xf7;
	}
  ioctl(serfd, EXSER_WRITE_REG, &exser_reg);

  if( flowcontrol == FLOW_RTSCTS || flowcontrol == FLOW_NONE ){
		/* Set hardware handshaking */
		exser_reg.reg = EFR_850;
		ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read EFR contents */
		if( flowcontrol == FLOW_RTSCTS)exser_reg.regvalue |= 0xc0;		/* Enable Auto hardware handshake */
		else exser_reg.regvalue = 0;		/* none flow control */
																	/* Enable Special function regs */
		ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		
		exser_reg.reg = UART_LCR;
		exser_reg.regvalue = lcr;
		ioctl(serfd, EXSER_WRITE_REG, &exser_reg);	/* Access the General registers */
		
		exser_reg.reg = UART_MCR;
		ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read MCR contents */
		exser_reg.regvalue |= 0x3;	/* Enable RTS/DTR hardware handshake */
		ioctl(serfd, EXSER_WRITE_REG, &exser_reg);

		exser_reg.reg = UART_LCR;
		exser_reg.regvalue = 0xbf;
		ioctl(serfd, EXSER_WRITE_REG, &exser_reg);	/* Access the special function registers */
	
		if(txfifoflg | rxfifoflg){
			/* Set the trigger levels */
			exser_reg.reg = FCTR;
			ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read FCTR contents */
			exser_reg.regvalue |= 0xb0;		/* Table D selecet, Tx prog. trig level */
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
			
			if( txfifoflg){
				exser_reg.reg = TRG_850;
				exser_reg.regvalue = txtriglvl;		/* Tx trigger level from dialog box */
				ioctl(serfd, EXSER_WRITE_REG, &exser_reg);				
			}
			
			if( rxfifoflg ){
				exser_reg.reg = FCTR;
				ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read FCTR contents */
				exser_reg.regvalue &= 0x7f;		/* Table D select, Rx prog. trig level */
				exser_reg.regvalue |= 0x30;		/* Table D select, Rx prog. trig level */
				ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
			
				exser_reg.reg = TRG_850;
				exser_reg.regvalue = rxtriglvl;		/* Rx trigger level from dialog box */
				ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
			}
		}
		
		/* Set the Hysterisis */
		exser_reg.reg = FCTR;
		ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read FCTR contents */
		exser_reg.regvalue &= 0xfc ;		/* clear the hyst */
		
		if( (flowcontrol == FLOW_RTSCTS ) && rtshystflg){
			exser_reg.regvalue |= ( rtshyst & 3) ;		/* set the hyst */
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
			
			exser_reg.reg = UART_LCR;
			exser_reg.regvalue = lcr;
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);	/* Access the General registers */
			
			exser_reg.reg = FCTR;
			ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read FCTR contents */
			exser_reg.regvalue |= 0x40;		/* Enable EMSR */
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);

			
			exser_reg.reg = EMSR_750;
			ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read EMSR contents */
			exser_reg.regvalue &= 0xcf;		/* clear the hyst */
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		
			exser_reg.regvalue |= ( ( rtshyst & 0xc) << 2 );		/* set the hyst */
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);	/* Write the Hysteresis */
			
			exser_reg.reg = FCTR;
			ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read FCTR contents */
			exser_reg.regvalue &= 0xbf;		/* Disable EMSR */
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		}	
	}
	
	if( flowcontrol == FLOW_XONXOFF ){
	/* Set XON/XOFF trigger and hysterisis */
		exser_reg.reg = EFR_850;
		ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read EFR contents */
		exser_reg.regvalue &= 0x3f;		/* Disable hardware handshake */
		exser_reg.regvalue |= 0xa;		/* Enable Tx/Rx XON1/XOFF1 */		
		ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		
		exser_reg.reg = XON1_850;
		exser_reg.regvalue = XON;		/* Set XON1 */		
		ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		
		exser_reg.reg = XOFF1_850;
		exser_reg.regvalue = XOFF;		/* Set XOFF1 */		
		ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		
		if(txfifoflg | rxfifoflg){
			/* Set the trigger levels */
			exser_reg.reg = FCTR;
			ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read FCTR contents */
			exser_reg.regvalue |= 0xb0;		/* Table D selecet, Tx prog. trig level */
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
			
			if( txfifoflg){
				exser_reg.reg = TRG_850;
				exser_reg.regvalue = txtriglvl;		/* Tx trigger level from dialog box */
				ioctl(serfd, EXSER_WRITE_REG, &exser_reg);				
			}
			
			if( rxfifoflg ){
				exser_reg.reg = FCTR;
				ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read FCTR contents */
				exser_reg.regvalue &= 0x7f;		/* Table D select, Rx prog. trig level */
				exser_reg.regvalue |= 0x30;		/* Table D select, Rx prog. trig level */
				ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
			
				exser_reg.reg = TRG_850;
				exser_reg.regvalue = rxtriglvl;		/* Rx trigger level from dialog box */
				ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
			}
		}
	
		/* Set the Hysterisis */
		exser_reg.reg = FCTR;
		ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read FCTR contents */
		exser_reg.regvalue &= 0xfc ;		/* clear the hyst */
		
		if( (flowcontrol == 0) && rtshystflg){
			exser_reg.regvalue |= ( rtshyst & 3) ;		/* set the hyst */
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
			
			exser_reg.reg = UART_LCR;
			exser_reg.regvalue = lcr;
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);	/* Access the General registers */
			
			exser_reg.reg = FCTR;
			ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read FCTR contents */
			exser_reg.regvalue |= 0x40;		/* Enable EMSR */
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);

			
			exser_reg.reg = EMSR_750;
			ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read EMSR contents */
			exser_reg.regvalue &= 0xcf;		/* clear the hyst */
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		
			exser_reg.regvalue |= ( ( rtshyst & 0xc) << 2 );		/* set the hyst */
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);	/* Write the Hysteresis */
			
			exser_reg.reg = FCTR;
			ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read FCTR contents */
			exser_reg.regvalue &= 0xbf;		/* Disable EMSR */
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		}	
	}
		
	exser_reg.reg = UART_LCR;
	exser_reg.regvalue = lcr;
	ioctl(serfd, EXSER_WRITE_REG, &exser_reg);	/* Access the General registers */
}

void ExserDoc::XR16C850Init()
{
	struct exser_read_reg exser_reg;
	int lcr;
		
	QMessageBox::information(0, "Exser", "Initializing XR16C850 UART");

	/* Access special function registers */
	exser_reg.reg = UART_LCR;
	ioctl(serfd, EXSER_READ_REG, &exser_reg);
	lcr = exser_reg.regvalue;		/* store the previous LCR reg. value */
	
	exser_reg.regvalue = 0xbf;	/* Access the special function registers */
	ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
	
	/* Set Auto RS485 Enable */
	exser_reg.reg = FCTR;
	ioctl(serfd, EXSER_READ_REG, &exser_reg);
	if( autorsflg ){
		exser_reg.regvalue |= 0x8;
	}
	else {
		exser_reg.regvalue &= 0xf7;
	}
  ioctl(serfd, EXSER_WRITE_REG, &exser_reg);

  if( flowcontrol == FLOW_RTSCTS || flowcontrol == FLOW_NONE ){
		/* Set hardware handshaking */
		exser_reg.reg = EFR_850;
		ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read EFR contents */
		if( flowcontrol == FLOW_RTSCTS)exser_reg.regvalue |= 0xc0;		/* Enable Auto hardware handshake */
		else exser_reg.regvalue = 0;		/* none flow control */
																	/* Enable Special function regs */
		ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		
		exser_reg.reg = UART_LCR;
		exser_reg.regvalue = lcr;
		ioctl(serfd, EXSER_WRITE_REG, &exser_reg);	/* Access the General registers */
		
		exser_reg.reg = UART_MCR;
		ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read MCR contents */
		exser_reg.regvalue |= 0x3;	/* Enable RTS/DTR hardware handshake */
		ioctl(serfd, EXSER_WRITE_REG, &exser_reg);

		exser_reg.reg = UART_LCR;
		exser_reg.regvalue = 0xbf;
		ioctl(serfd, EXSER_WRITE_REG, &exser_reg);	/* Access the special function registers */
	
		if(txfifoflg | rxfifoflg){
			/* Set the trigger levels */
			exser_reg.reg = FCTR;
			ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read FCTR contents */
			exser_reg.regvalue |= 0xb0;		/* Table D selecet, Tx prog. trig level */
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
			
			if( txfifoflg){
				exser_reg.reg = TRG_850;
				exser_reg.regvalue = txtriglvl;		/* Tx trigger level from dialog box */
				ioctl(serfd, EXSER_WRITE_REG, &exser_reg);				
			}
			
			if( rxfifoflg ){
				exser_reg.reg = FCTR;
				ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read FCTR contents */
				exser_reg.regvalue &= 0x7f;		/* Table D select, Rx prog. trig level */
				exser_reg.regvalue |= 0x30;		/* Table D select, Rx prog. trig level */
				ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
			
				exser_reg.reg = TRG_850;
				exser_reg.regvalue = rxtriglvl;		/* Rx trigger level from dialog box */
				ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
			}
		}
		
		/* Set the Hysterisis */
		exser_reg.reg = FCTR;
		ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read FCTR contents */
		exser_reg.regvalue &= 0xfc ;		/* clear the hyst */
		
		if( (flowcontrol == FLOW_RTSCTS ) && rtshystflg){
			exser_reg.regvalue |= rtshyst ;		/* set the hyst */
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		}	
	}
	
	if( flowcontrol == FLOW_XONXOFF ){
	/* Set XON/XOFF trigger and hysterisis */
		exser_reg.reg = EFR_850;
		ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read EFR contents */
		exser_reg.regvalue &= 0x3f;		/* Disable hardware handshake */
		exser_reg.regvalue |= 0xa;		/* Enable Tx/Rx XON1/XOFF1 */		
		ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		
		exser_reg.reg = XON1_850;
		exser_reg.regvalue = XON;		/* Set XON1 */		
		ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		
		exser_reg.reg = XOFF1_850;
		exser_reg.regvalue = XOFF;		/* Set XOFF1 */		
		ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		
		if(txfifoflg | rxfifoflg){
			/* Set the trigger levels */
			exser_reg.reg = FCTR;
			ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read FCTR contents */
			exser_reg.regvalue |= 0xb0;		/* Table D selecet, Tx prog. trig level */
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
			
			if( txfifoflg){
				exser_reg.reg = TRG_850;
				exser_reg.regvalue = txtriglvl;		/* Tx trigger level from dialog box */
				ioctl(serfd, EXSER_WRITE_REG, &exser_reg);				
			}
			
			if( rxfifoflg ){
				exser_reg.reg = FCTR;
				ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read FCTR contents */
				exser_reg.regvalue &= 0x7f;		/* Table D select, Rx prog. trig level */
				exser_reg.regvalue |= 0x30;		/* Table D select, Rx prog. trig level */
				ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
			
				exser_reg.reg = TRG_850;
				exser_reg.regvalue = rxtriglvl;		/* Rx trigger level from dialog box */
				ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
			}
		}
	
		/* Set the Hysterisis */
		exser_reg.reg = FCTR;
		ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read FCTR contents */
		exser_reg.regvalue &= 0xfc ;		/* clear the hyst */
		
		if( (flowcontrol == 0) && rtshystflg){
			exser_reg.regvalue |= rtshyst ;		/* set the hyst */
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		}	
	}
		
	exser_reg.reg = UART_LCR;
	exser_reg.regvalue = lcr;
	ioctl(serfd, EXSER_WRITE_REG, &exser_reg);	/* Access the General registers */
}

void ExserDoc::XR17C158Init()
{
	struct exser_read_reg exser_reg;

	QMessageBox::information(0, "Exser", "Initializing XR17C158 UART");

	/* Set Auto RS485 Enable */
	exser_reg.reg = FCTR_158;
	ioctl(serfd, EXSER_READ_REG, &exser_reg);
	if( autorsflg){
		exser_reg.regvalue |= 0x20;
	}
	else {
		exser_reg.regvalue &= 0xdf;
	}
  ioctl(serfd, EXSER_WRITE_REG, &exser_reg);


  if( flowcontrol == FLOW_RTSCTS || flowcontrol == FLOW_DTRDSR ){
		/* Set hardware handshaking */
		exser_reg.reg = EFR_158;
		ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read EFR contents */
		exser_reg.regvalue |= 0xd0;		/* Enable Auto hardware handshake */
																	/* Enable Special function regs */
		ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		
		exser_reg.reg = UART_MCR;
		ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read MCR contents */
		if( flowcontrol == FLOW_DTRDSR ) exser_reg.regvalue |= 0x4;	/* Enable DTR/DSR hardware handshake */
		else exser_reg.regvalue &= 0xfb;									/* Enable RTS/CTS hardware handshake */
		ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		
		if(txfifoflg | rxfifoflg){
			/* Set the trigger levels */
			exser_reg.reg = FCTR_158;
			ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read FCTR contents */
			exser_reg.regvalue |= 0xc0;		/* Table D selecet */
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
			
			if( txfifoflg){
				exser_reg.reg = TXTRG_158;
				exser_reg.regvalue = txtriglvl;		/* Tx trigger level from dialog box */
				ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
			}
			
			if( rxfifoflg ){
				exser_reg.reg = RXTRG_158;
				exser_reg.regvalue = rxtriglvl;		/* Rx trigger level from dialog box */
				ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
			}
		}
		
		exser_reg.reg = EFR_158;
		ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read EFR contents */
		exser_reg.regvalue &= 0xe0;		/* Disable Special function regs */
		ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		
		/* Set the Hysterisis */
		exser_reg.reg = FCTR_158;
		ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read FCTR contents */
		exser_reg.regvalue &= 0xf0 ;		/* clear the hyst */
		
		if( (flowcontrol == FLOW_RTSCTS ) && rtshystflg){
			exser_reg.regvalue |= iahyst[rtshyst] ;		/* set the hyst */
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		}
		if( (flowcontrol == FLOW_DTRDSR ) && dtrhystflg){
			exser_reg.regvalue |= iahyst[dtrhyst] ;		/* set the hyst */
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		}		
	}
	
	if( flowcontrol == FLOW_XONXOFF ){
	/* Set XON/XOFF trigger and hysterisis */
		exser_reg.reg = EFR_158;
		ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read EFR contents */
		exser_reg.regvalue &= 0x30;		/* Disable hardware handshake */
		exser_reg.regvalue |= 0x1a;		/* Enable Tx/Rx XON1/XOFF1 */		
		ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		
		exser_reg.reg = XON1_158;
		exser_reg.regvalue = XON;		/* Set XON1 */		
		ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		
		exser_reg.reg = XOFF1_158;
		exser_reg.regvalue = XOFF;		/* Set XOFF1 */		
		ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		
		if(txfifoflg | rxfifoflg){
			/* Set the trigger levels */
			exser_reg.reg = FCTR_158;
			ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read FCTR contents */
			exser_reg.regvalue |= 0xc0;		/* Table D selecet */
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
			
			if( txfifoflg){
				exser_reg.reg = TXTRG_158;
				exser_reg.regvalue = txtriglvl;		/* Tx trigger level from dialog box */
				ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
			}
			
			if( rxfifoflg ){
				exser_reg.reg = RXTRG_158;
				exser_reg.regvalue = rxtriglvl;		/* Rx trigger level from dialog box */
				ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
			}
		}
	
		exser_reg.reg = EFR_158;
		ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read EFR contents */
		exser_reg.regvalue &= 0xef;		/* Disable Special function regs */
		ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		
		/* Set the Hysterisis */
		exser_reg.reg = FCTR_158;
		ioctl(serfd, EXSER_READ_REG, &exser_reg);	/* Read FCTR contents */
		exser_reg.regvalue &= 0xf0 ;		/* clear the hyst */
		
		if( rtshystflg ){
			exser_reg.regvalue |= iahyst[rtshyst] ;		/* set the hyst */
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		}
		if( dtrhystflg){
			exser_reg.regvalue |= iahyst[dtrhyst] ;		/* set the hyst */
			ioctl(serfd, EXSER_WRITE_REG, &exser_reg);
		}		
		
	}
}

void ExserDoc::ComRead()
{
	int i;
	
	i = read( serfd, rdBuf + noBytes, 256);
	
	if( TestingFlg ){
	/*	Testing is going on */
		if( RxBytes + i > TxBytes)RxBytes = 0;	/* Invalidate the rx packet */
		else {
			/* copy the received bytes to the receiver buffer */
			memcpy( RxBuf + RxBytes, rdBuf + noBytes, i);
			RxBytes += i; /* Update the number of rxbytes */
		}
	}
	
	if( (noBytes + i) >= (SERIAL_BUF_SIZE - 256)){
		memcpy( rdBuf, rdBuf + ((noBytes + i) -
													 (SERIAL_BUF_SIZE-256)),
													  SERIAL_BUF_SIZE - 256);
		noBytes = SERIAL_BUF_SIZE - 256;
	}
	else noBytes += i;
	
	rdBuf[noBytes] = '\0';
	
	/* Update the views */
	updateAllViews ( 0 );
}

void ExserDoc::SetSerParm()
{
	
  SerParmDlg* dlg= new SerParmDlg();
	
  if( UARTType != XR17C158){
		dlg->FlowControl->removeItem( 3 );	/* remove the DTR Hardware handshake */

  	dlg->DtrHyst->clear();		/* clear the list of items */
	  dlg->DtrHyst->insertItem("None", -1 );

  	dlg->RtsHyst->clear();		/* Clear the list of items */
  }

  switch( UARTType ){
	case PORT_16550A:	
		ST550SetParm( dlg );
		break;

  case ST16C650:
  	ST650SetParm( dlg );
  	break;
  	
  case XR16C750:
  	XR750SetParm( dlg );
	  break;
	
	case XR16C850:
		XR850SetParm( dlg );
		break;
		
	case XR17C158:
		break;
		
	default:
		STD8250SetParm( dlg );
		break;
	}
  	
	dlg->BaudRate->setCurrentItem( baudrate );
	dlg->DataBits->setCurrentItem( databits );
	dlg->Parity->setCurrentItem( parity );
	dlg->StopBits->setCurrentItem( stopbits );
	dlg->FlowControl->setCurrentItem( flowcontrol);

  dlg->DtrHystFlg->setChecked( dtrhystflg );
  dlg->RtsHystFlg->setChecked( rtshystflg );
  dlg->DtrHyst->setCurrentItem( dtrhyst );
  dlg->RtsHyst->setCurrentItem( rtshyst );
  dlg->TxFIFOFlg->setChecked( txfifoflg );
  dlg->RxFIFOFlg->setChecked( rxfifoflg );
  dlg->TxTrigLvl->setCurrentItem( txtriglvl );
  dlg->RxTrigLvl->setCurrentItem( rxtriglvl );

  dlg->IntLoopBack->setChecked( intloopb );
  dlg->AutoRs->setChecked( autorsflg );

  if(dlg->exec()){
		baudrate = dlg->BaudRate->currentItem( );
		databits = dlg->DataBits->currentItem( );
		parity = dlg->Parity->currentItem( );
		stopbits = dlg->StopBits->currentItem( );
		flowcontrol = dlg->FlowControl->currentItem( );
	
	  dtrhystflg = dlg->DtrHystFlg->isChecked( );
	  rtshystflg = dlg->RtsHystFlg->isChecked( );
	  dtrhyst = dlg->DtrHyst->currentItem( );
	  rtshyst = dlg->RtsHyst->currentItem( );
	
	  txfifoflg = dlg->TxFIFOFlg->isChecked( );
	  rxfifoflg = dlg->RxFIFOFlg->isChecked( );
	  txtriglvl = dlg->TxTrigLvl->currentItem( );
  	rxtriglvl = dlg->RxTrigLvl->currentItem( );
  	
  	intloopb = dlg->IntLoopBack->isChecked();
  	autorsflg = dlg->AutoRs->isChecked();
  	
  	/* Set the serial port parameters */
  	SetSerialParm();
  }
}

void ExserDoc::STD8250SetParm( SerParmDlg* dlg )
{
	/* Set the trigger level list */
  dlg->RtsHyst->insertItem( "None", -1 );

  /* Clear the Tx trigger levels */
	dlg->TxTrigLvl->clear( );
	dlg->TxTrigLvl->insertItem( "None", -1 );
	
  /* Clear the Rx trigger levels */
	dlg->RxTrigLvl->clear( );
	dlg->RxTrigLvl->insertItem( "None", -1 );
}


void ExserDoc::ST550SetParm( SerParmDlg* dlg )
{
	QString str;
	
	/* Set the trigger level list */
  dlg->RtsHyst->insertItem( "None", -1 );

  /* Clear the Tx trigger levels */
	dlg->TxTrigLvl->clear( );
	dlg->TxTrigLvl->insertItem( "None", -1 );

  /* Clear the Rx trigger levels */
	dlg->RxTrigLvl->clear( );
	/* Set the TX trigger levels */
	dlg->RxTrigLvl->insertItem( str.number( 1 ), -1 );
	dlg->RxTrigLvl->insertItem( str.number( 4 ), -1 );
	dlg->RxTrigLvl->insertItem( str.number( 8 ), -1 );
	dlg->RxTrigLvl->insertItem( str.number( 14 ), -1 );
}

void ExserDoc::ST650SetParm( SerParmDlg* dlg )
{
	QString str;
	
	/* Set the trigger level list */
  dlg->RtsHyst->insertItem( str.number( 0 ), -1 );

  /* Clear the Tx trigger levels */
	dlg->TxTrigLvl->clear( );
	/* Set the TX trigger levels */
	dlg->TxTrigLvl->insertItem( str.number( 16 ), -1 );
	dlg->TxTrigLvl->insertItem( str.number( 8 ), -1 );
	dlg->TxTrigLvl->insertItem( str.number( 24 ), -1 );
	dlg->TxTrigLvl->insertItem( str.number( 30 ), -1 );

  /* Clear the Rx trigger levels */
	dlg->RxTrigLvl->clear( );
	/* Set the TX trigger levels */
	dlg->RxTrigLvl->insertItem( str.number( 8 ), -1 );
	dlg->RxTrigLvl->insertItem( str.number( 16 ), -1 );
	dlg->RxTrigLvl->insertItem( str.number( 24 ), -1 );
	dlg->RxTrigLvl->insertItem( str.number( 28 ), -1 );
}

void ExserDoc::XR750SetParm( SerParmDlg* dlg )
{
	int i;
	QString str;
	
	/* Set the Hyst list */
	for( i = 0; i < 16; i++ ) {
		dlg->RtsHyst->insertItem( str.number( i ), -1 );
	}
  /* Clear the Tx trigger levels */
	dlg->TxTrigLvl->clear( );
	/* Set the TX trigger levels */
	for( i = 0; i < 64; i++ ) {
		dlg->TxTrigLvl->insertItem( str.number( i ), -1 );
	}

  /* Clear the Rx trigger levels */
	dlg->RxTrigLvl->clear( );
	/* Set the TX trigger levels */
	for( i = 1; i < 65; i++ ) {
		dlg->RxTrigLvl->insertItem( str.number( i ), -1 );
	}
}

const char *RtsHyst850[] = {
	"Next Trig.lvl",
	"4char+trig.lvl",
	"6char+trig.lvl",
	"8char+trig.lvl",
	NULL
};

void ExserDoc::XR850SetParm( SerParmDlg* dlg )
{
	int i;
	QString str;
	
	/* Set the trigger level list */
  dlg->RtsHyst->insertStrList( RtsHyst850, 4, -1 );

  /* Clear the Tx trigger levels */
	dlg->TxTrigLvl->clear( );
	/* Set the TX trigger levels */
	for( i = 0; i < 128; i++ ) {
		dlg->TxTrigLvl->insertItem( str.number( i ), -1 );
	}

  /* Clear the Rx trigger levels */
	dlg->RxTrigLvl->clear( );
	/* Set the TX trigger levels */
	for( i = 1; i < 129; i++ ) {
		dlg->RxTrigLvl->insertItem( str.number( i ), -1 );
	}
}

void ExserDoc::SetSerTest()
{
	SerTestDlg* dlg= new SerTestDlg();
	
	dlg->Continuous->setChecked( ContFlg );
	dlg->StopOnError->setChecked( StopOnErrFlg );
	dlg->NoPack->setChecked( NoPackFlg );
	
	dlg->QuickBrown->setChecked( testpat == QUICK_BROWN);
	dlg->Alphabets->setChecked( testpat == ALPHABETS);
	dlg->UserPattern->setChecked( testpat == USER_PAT);
	dlg->File->setChecked( testpat == FILE_PAT);
	
	if(dlg->exec()){
		
		ContFlg = dlg->Continuous->isChecked();
		StopOnErrFlg = dlg->StopOnError->isChecked();
		NoPackFlg = dlg->NoPack->isChecked();
		if( NoPackFlg ) NoPack = atol( dlg->NoPackEdit->text() );
		
		if(dlg->QuickBrown->isChecked()){
			testpat = QUICK_BROWN;
			PatLen = sizeof( QuickBrown)  - 1;
			memcpy( TxBuf, QuickBrown, sizeof( QuickBrown) );	
		}
		if(dlg->Alphabets->isChecked()){
			testpat = ALPHABETS;
			PatLen = sizeof( Alphabet )  - 1;
			memcpy( TxBuf, Alphabet, sizeof( Alphabet) );	
		}
		if(dlg->UserPattern->isChecked()){
			testpat = USER_PAT;
			QString userpat = dlg->UserPatternEdit->text();
			PatLen = userpat.length();
			memcpy( TxBuf, userpat, PatLen);	
		}
		if(dlg->File->isChecked()){
			TestFlName = QFileDialog::getOpenFileName(0,0);
			if( -1 ==(testfd = open( TestFlName, O_RDONLY ))){
				QMessageBox::information(0, "Exser", "Unable to open the file");
				testpat = 0;
			}
			else {
				testpat = FILE_PAT;
				close( testfd );testfd = -1;
			}
		}
	}	
}

void ExserDoc::SerRunTest()
{
	if( TestingFlg == -1 && ( testpat ) ){
		if( testpat == FILE_PAT)testfd = open( TestFlName, O_RDONLY );
		TimerID =	startTimer( 200 );
		TestingFlg = 1;TxBytes = RxBytes = 0;
	}
}

void ExserDoc::SerStopTest()
{
	if( TestingFlg == 1){
		killTimer( TimerID );
		TestingFlg = -1;
		TimerID = -1;
		if( testfd != -1){
			close( testfd );testfd = -1;
		}
	}
}

void ExserDoc::timerEvent( QTimerEvent *e )
{
	ExserView *w;
	
	if( TxBytes != RxBytes)packerr++;	/* All tx bytes are not received */
	else {
		if(memcmp( TxBuf, RxBuf, TxBytes) != 0)packerr++; /* tx != rx */
	}
	
	w=pViewList->first();
	w->setCaption(QString(m_title+":%1  %2").arg(noTx).arg(packerr));	

	if( NoPackFlg && (noTx == NoPack)){
		SerStopTest();
		QMessageBox::information( 0, "Exser", "Stop on No packets");
		return;
	}
	if( packerr && StopOnErrFlg){
		SerStopTest();
		QMessageBox::information( 0, "Exser", "Stop on error");
		return;
	}
	RxBytes = 0;	/* Clear the read buffer */
	
	switch( testpat ){
	case QUICK_BROWN:
	case ALPHABETS:
	case USER_PAT:
		TxBytes = write( serfd, TxBuf, PatLen);
		noTx ++;
		break;
		
	case FILE_PAT:
		TxBytes = read( testfd, TxBuf, 100);
		if( TxBytes == 0){
			/* We reached the end of file */
			close( testfd);testfd = -1;
			if(ContFlg){
				/* continuous flag is set */
				if( -1 ==(testfd = open( TestFlName, O_RDONLY ))){
					QMessageBox::information(0, "Exser", "Unable to open the file");
					SerStopTest();
				}
				else TxBytes = read( testfd, TxBuf, 100);
			}
			else SerStopTest();
		}
		
		if( testfd != -1){
			write( serfd, TxBuf, TxBytes );
			noTx ++;
		}
		break;
		
	default:
		SerStopTest();
		QMessageBox::information( 0, "Exser", "error test pattern");
		break;
	}	
		
}
